/*
Copyright (c) 2009 by Matjaz Kovac

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do 
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
SOFTWARE.
*/

/*

This program reads one or more image files and writes the appropriate 
C declarations to the standard output.
	
COMPILATION:

	gcc bmap2c.cpp -lroot -lbe -ltranslation -s

USAGE:
	
Suppose we have an image named 'pic.png'. First generate a C header with:
	
	bmap2c pic.png > bitmaps.h
	
Then in your program #include "bitmaps.h" and somewhere in your code do:

	BBitmap *bmap = new BBitmap(BRect(0,0,pic_png_width, pic_png_height), B_RGBA32);
	bmap->SetBits(pic_png_bits, pic_png_size, 0, B_RGBA32);
		
NOTES:
	
* Filename is used for variable prefixes. Spaces, '.' and '-' are replaced with '_' but you
need to have C identifier-like file names.
* Since the Translation Kit always translates to its intermediary format, the bitmap 
color space is always B_RGBA32 even for GIFs etc.
* To see the alpha channel in action do SetDrawingMode(B_OP_ALPHA).
* BRect Width() and Height() are used for _width and _height variables, therefore do *not* substract 1 from those values. See The BeBook section "The Coordinate Space" for the explanation on why this actually makes perfect sense.

*/

#include <Application.h>
#include <TranslationKit.h>
#include <File.h>
#include <Path.h>
#include <Bitmap.h>
#include <String.h>
#include <unistd.h>
#include <stdio.h>

#define VERSION "0.5"
#define HELP_TEXT "bmap2c v"VERSION" by Matjaž Kovač.\nConverts images to C declarations.\nUSAGE:bmap2c [-b] [-d] [-w] [-h] {filename}\n-b output byte list\n-d decimal\n-w wrap lines\n"

using namespace std;

// List as bytes, longs otherwise.
bool bytes = false;
// Write decimal strings, hex otherwise.
bool decimal = false;
// Insert LF after linewrap items.
int linewrap = 0;


/*
	Outputs C header declarations.
	Plain old printf() is used to make compilations
	on both Haiku and BeOS hassle-free. 
*/
int convert_image(const char *filename)
{
	BFile file(filename, B_READ_ONLY);
	BTranslatorRoster *roster = BTranslatorRoster::Default();
	BBitmapStream stream;
	BBitmap *bmap = NULL;
	if (roster->Translate(&file, NULL, NULL, &stream, B_TRANSLATOR_BITMAP) == B_OK) {
		stream.DetachBitmap(&bmap);
		// Translation succeeded, write header defs.
		BPath path(filename);
		BString name(path.Leaf());
		name.ReplaceAll('.','_');
		name.ReplaceAll('-','_');
		name.ReplaceAll(' ','_');
		printf("/* generated by bmap2c v"VERSION" */\n");
		printf("int %s_width = %d;\n", name.String(), bmap->Bounds().IntegerWidth());
		printf("int %s_height = %d;\n", name.String(), bmap->Bounds().IntegerHeight());
		printf("int %s_size = %d;\n", name.String(), bmap->BitsLength());
		if (bytes)
			printf("uint8 %s_bits[] = {\n", name.String());
		else
			printf("uint32 %s_bits[] = {\n", name.String());
		uint8 *bits8 = (uint8*)bmap->Bits();
		uint32 *bits32 = (uint32*)bmap->Bits();
		int bpr = bmap->BytesPerRow();
		if (!bytes)
			bpr /= 4;
		int n = 0;
		// process line-by-line
		for (int y = 0; y <= bmap->Bounds().IntegerHeight(); y++) {
			for (int i = 0; i < bpr; i++) {
				if (bytes)
					printf(decimal ? "%u, ":"0x%x, ", *(bits8++));
				else	
					printf(decimal ? "%uUL, ":"0x%x, ", *(bits32++));
				n++;
				if (linewrap > 0 && (n % linewrap == 0))
					printf("\n");
			}
			if (linewrap < 1)
				printf("\n");
		}
		printf("};\n\n");
		delete bmap;
		return 0;
	}
	else {
		fprintf(stderr, "Cannot load '%s'.\n", filename);
		return 1;
	}
}


int main(int argc, char **argv) 
{
	// a connection to app_server is needed
	new BApplication("application/x-vnd.mk-bmap2c");

	// parse option args
	int c;
	while ((c = getopt(argc,argv, "dbwh")) != -1)
		switch (c) {
			case 'b':
				bytes = true;
				break;
			case 'd':
				decimal = true;				
				break;
			case 'w':
				linewrap = 16;
				break;
			case 'h':
				printf(HELP_TEXT);
				break;
			default:
				return 1;
		}
	
	// process non-option arguments
	for (int i = optind; i < argc; i++)
		convert_image(argv[i]);
		
	return 0;
}
